@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2013 The Music Player Daemon Project
+ * Copyright (C) 2003-2014 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -19,62 +19,19 @@
 
 #include "config.h"
 #include "LogV.hxx"
-#include "ConfigData.hxx"
-#include "ConfigGlobal.hxx"
-#include "ConfigOption.hxx"
-#include "system/fd_util.h"
-#include "system/FatalError.hxx"
-#include "fs/Path.hxx"
-#include "fs/FileSystem.hxx"
 #include "util/Error.hxx"
-#include "util/Domain.hxx"
-#include "system/FatalError.hxx"
-
-#include <glib.h>
 
 #include <assert.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <string.h>
-#include <fcntl.h>
 #include <stdio.h>
-#include <stdlib.h>
-#include <time.h>
-#include <unistd.h>
+#include <string.h>
 #include <errno.h>
 
-static GLogLevelFlags
-ToGLib(LogLevel level)
-{
-	switch (level) {
-	case LogLevel::DEBUG:
-		return G_LOG_LEVEL_DEBUG;
-
-	case LogLevel::INFO:
-		return G_LOG_LEVEL_INFO;
-
-	case LogLevel::DEFAULT:
-		return G_LOG_LEVEL_MESSAGE;
-
-	case LogLevel::WARNING:
-	case LogLevel::ERROR:
-		return G_LOG_LEVEL_WARNING;
-	}
-
-	assert(false);
-	gcc_unreachable();
-}
-
-void
-Log(const Domain &domain, LogLevel level, const char *msg)
-{
-	g_log(domain.GetName(), ToGLib(level), "%s", msg);
-}
-
 void
 LogFormatV(const Domain &domain, LogLevel level, const char *fmt, va_list ap)
 {
-	g_logv(domain.GetName(), ToGLib(level), fmt, ap);
+	char msg[1024];
+	vsnprintf(msg, sizeof(msg), fmt, ap);
+	Log(domain, level, msg);
 }
 
 void
@@ -159,7 +116,7 @@ FormatError(const Error &error, const char *fmt, ...)
 void
 LogErrno(const Domain &domain, int e, const char *msg)
 {
-	LogFormat(domain, LogLevel::ERROR, "%s: %s", msg, g_strerror(e));
+	LogFormat(domain, LogLevel::ERROR, "%s: %s", msg, strerror(e));
 }
 
 void
