@@ -56,8 +56,18 @@ NfsFileReader::Close()
 		return;
 	}
 
+	/* this cancels State::MOUNT */
 	connection->RemoveLease(*this);
 
+	CancelOrClose();
+}
+
+void
+NfsFileReader::CancelOrClose()
+{
+	assert(state != State::INITIAL &&
+	       state != State::DEFER);
+
 	if (state == State::IDLE)
 		/* no async operation in progress: can close
 		   immediately */
@@ -164,6 +174,8 @@ NfsFileReader::OnNfsConnectionFailed(const Error &error)
 {
 	assert(state == State::MOUNT);
 
+	state = State::INITIAL;
+
 	Error copy;
 	copy.Set(error);
 	OnNfsFileError(std::move(copy));
@@ -174,7 +186,7 @@ NfsFileReader::OnNfsConnectionDisconnected(const Error &error)
 {
 	assert(state > State::MOUNT);
 
-	state = State::INITIAL;
+	CancelOrClose();
 
 	Error copy;
 	copy.Set(error);
@@ -246,6 +258,30 @@ NfsFileReader::OnNfsCallback(unsigned status, void *data)
 void
 NfsFileReader::OnNfsError(Error &&error)
 {
+	switch (state) {
+	case State::INITIAL:
+	case State::DEFER:
+	case State::MOUNT:
+	case State::IDLE:
+		assert(false);
+		gcc_unreachable();
+
+	case State::OPEN:
+		connection->RemoveLease(*this);
+		state = State::INITIAL;
+		break;
+
+	case State::STAT:
+		connection->RemoveLease(*this);
+		connection->Close(fh);
+		state = State::INITIAL;
+		break;
+
+	case State::READ:
+		state = State::IDLE;
+		break;
+	}
+
 	OnNfsFileError(std::move(error));
 }
 
