@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2013 The Music Player Daemon Project
+ * Copyright (C) 2003-2014 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -21,7 +21,7 @@
 #include "SocketError.hxx"
 #include "util/Domain.hxx"
 
-#include <glib.h>
+#include <string.h>
 
 const Domain socket_domain("socket");
 
@@ -41,6 +41,6 @@ SocketErrorMessage::SocketErrorMessage(socket_error_t code)
 #else
 
 SocketErrorMessage::SocketErrorMessage(socket_error_t code)
-	:msg(g_strerror(code)) {}
+	:msg(strerror(code)) {}
 
 #endif
